import veo
import os
from cffi import FFI

print("\nVEO test:")
print("Pass a cffi built structure to the VE as argument on the stack.")
print("Sum the elements and multiply with a factor. Correct result is 30.")
print("\n")

p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvetest6.so")
c = p.open_context()

ffi = FFI()
ffi.cdef("""
    struct abc {
        int a, b, c;
    };
    """)

abc = ffi.new("struct abc *")
abc.a = 1
abc.b = 2
abc.c = 3

# we'll pass the struct * as a void *
lib.multeach.args_type("void *", "int")
lib.multeach.ret_type("int")

req = lib.multeach(c, veo.OnStack(ffi.buffer(abc)), 5)
r = req.wait_result()
print("result = %r" % r)

del p
print("finished")



import veo
import os
import numpy as np


print """
VEO test:

Call a fortran function and pass in numpy arrays and parameters, by reference.

function test9(a, b, c, n, m)
    integer, dimension(n), intent(in) :: a
    integer, dimension(n) :: b
    double precision, intent(out) :: c(n,m)
    integer i, j, test9
    test9 = 0
    do i = 1, n
       test9 = test9 + a(i)
       b(i) = a(i) + 2
       do j = 1, m
          c(i,j) = dble(a(i)) * dble(j)
       end do
    end do
    return
end

a, b, c are numpy arrays, a, c are passed by reference, on the stack,
b is a buffer which was allocated separately on the VE and it's address
is passed as an argument. b and c are returned and printed.

"""


def round(n, r):
    return int(r * ((n + (r - 1)) / r))


p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvetest9.so")
f = lib.find_function("test9")
ctx = p.open_context()

n = 5; m = 3
a = np.array([1,2,3,4,5], dtype=np.int32)
b = np.array([0,0,0,0,0], dtype=np.int32)
c = np.zeros((n,m), dtype=np.float64, order="F")
nsz = np.array([a.size], dtype=np.int32)
msz = np.array([m], dtype=np.int32)
# this does not work
#nsz = np.int32(buff.size)

b_ve = p.alloc_mem( round(b.size * 4, 8) )
print("allocated b on VE: ", b_ve)

p.write_mem(b_ve, b, b.size * 4)
print("wrote b to VE (zeros).")

f.args_type("int *", "int *", "double *", "int *", "int *")
f.ret_type("int")

req = f(ctx, veo.OnStack(a, inout=veo.INTENT_IN),
        b_ve, veo.OnStack(c, inout=veo.INTENT_OUT),
        veo.OnStack(nsz), veo.OnStack(msz))

res = req.wait_result()

print("Request returns sum = ", res)

p.read_mem(b, b_ve, b.size * 4)
print("read b from VE.")

print("b after VEO call:", b)
print("c after VEO call:", c)

del p
print("finished")



import veo
import os
import time

def proc_hook(x):
    print("hello %r, this is the proc_init_hook!" % x)


print("\nVEO test:")
print("Call a trivial function as proc init_hook, call some trivial function to check that VEO works.")
print("Not opening a second VeoProc instance after the deletion of the first one due to a bug.")
print("\n")

veo.set_proc_init_hook(proc_hook)

p=veo.VeoProc(0)
lib=p.load_library(os.getcwd() + "/libvetest2.so")
f=lib.find_function("print_args")
c=p.open_context()

f.args_type("int", "unsigned int", "long", "unsigned long", "double", "float")
print(f)
args = (-12, 123456, -1234L, 1234567L, 0.12345, 0.12345)
print("sending args: %r" % (args,))
req=f(c, *args)
print(req)
r=req.wait_result()

del p
print("finished")



import veo
import os
import numpy as np


print("\nVEO test:")
print("Allocate a buffer on VE, copy a string to it.")
print("Display its address as seen on the VE and display the content as string.")
print("\n")

def round(n, r):
    return int(r * ((n + (r - 1)) / r))


p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvetest4.so")
c = p.open_context()

buff = np.frombuffer("Hello VE!!!\x00", dtype=np.uint8)

buff_ve = p.alloc_mem( round(buff.size, 8) )
print("allocated buffer on VE: ", buff_ve)

p.write_mem(buff_ve, buff, buff.size)
print("wrote buffer to VE.")

lib.print_buff.args_type("char *")
req = lib.print_buff(c, buff_ve)
print(req)
r = req.wait_result()

del p
print("finished")



import veo
import os
import time

print("\nVEO test:")
print("First function call passes 6 arguments of various types to VE, VE prints them.")
print("Second function passes 10 int arguments (on the stack), VE prints them.")
print("\n")

p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvetest2.so")
c = p.open_context()

args = (-12, 123456, -1234L, 1234567L, 0.12345, 0.12345)
print("sending args: %r" % (args,))
lib.print_args.args_type("int", "unsigned int", "long", "unsigned long", "double", "float")
req = lib.print_args(c, *args)
req.wait_result()

print("\nand now 10 arguments!\n")

fargs = ["int"] * 10
lib.print_args10.args_type(*fargs)
args = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
print("sending args: %r" % (args,))
req = lib.print_args10(c, *args)
req.wait_result()

del p
print("finished")



import veo
import os
import numpy as np


print("""
VEO test:
Pass an IN array and an INOUT array, the VE side sums up elements of IN array
and copies its values incremented by 2 into the second array.

Arguments are all passed by reference and their values are passed on the stack.
""")


def round(n, r):
    return int(r * ((n + (r - 1)) / r))

p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvetest8.so")
c = p.open_context()

buff = np.array([1,2,3,4,5], dtype=np.int32 )
buff2 = np.array([0,0,0,0,0], dtype=np.int32 )
nbuff = np.array([buff.size], dtype=np.int32 )
# this does not work
#nbuff = np.int32(buff.size)

lib.mod_buff.args_type("int *", "int *", "int *")
lib.mod_buff.ret_type("int")

req = lib.mod_buff(c, veo.OnStack(buff, inout=veo.INTENT_IN),
                   veo.OnStack(buff2, inout=veo.INTENT_INOUT),
                   veo.OnStack(nbuff))

res = req.wait_result()
print("request returns res = ", res)

print("buff2 after VEO call:", buff2)

del p
print("finished")



import veo
import os
import numpy as np


print("\nVEO test:")
print("Allocate a buffer on VE, copy a string asynchronously to it.")
print("Display its address as seen on the VE and display the content as string.")
print("Modify buffer on VE and retrieve the buffer asynchronously.")
print("Display the content of modified buffer as a string.\n")

def round(n, r):
    return int(r * ((n + (r - 1)) / r))


p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvetest7.so")
f = lib.find_function("print_mod_buff")
c = p.open_context()

buff = np.frombuffer("Hello VE!!!\x00", dtype=np.uint8)

buff_ve = p.alloc_mem( round(buff.size, 8) )
print("allocated buffer on VE: ", buff_ve)

r1 = c.async_write_mem(buff_ve, buff, buff.size)
print("async_write req:", r1)

f.args_type("char *")
r2 = f(c, buff_ve)
print("print_mod_buff req:", r2)

r3 = c.async_read_mem(buff, buff_ve, buff.size)
print("async_read req:", r3)

r = r3.wait_result()
print("VH buffer:", buff.tobytes())

del p
print("finished")



import veo
import os

print("\nVEO example:\nOpen a context and sleep 5s on the VE\n")

p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvesleep.so")
print("lib = ", lib)

lib.do_sleep.args_type("int")
print("lib.do_sleep = ", lib.do_sleep)

c = p.open_context()

req = lib.do_sleep(c, 5)
print("req = ", req)
print("result = ", req.wait_result())

print("deleting proc")
del p
print("finished")



import sys, glob
from distutils.core import setup
from distutils.extension import Extension
import numpy

USE_CYTHON = False
ext = ".c"
if "--use-cython" in sys.argv:
    from Cython.Build import cythonize
    sys.argv.remove("--use-cython")
    USE_CYTHON = True
    ext = ".pyx"


_ext_mods=[
    Extension("veo",
              sources=["src/veo" + ext],
              libraries=["veo"], # Unix-like specific
              library_dirs=["/opt/nec/ve/veos/lib64"],
              include_dirs=["/opt/nec/ve/veos/include", numpy.get_include()],
              extra_link_args=["-Wl,-rpath=/opt/nec/ve/veos/lib64"]
    ),
]

_example_files = glob.glob("./examples/*.py")
_example_files.extend(glob.glob("./examples/*.c"))
_example_files.extend(glob.glob("./examples/*.f90"))
_example_files.append("examples/Makefile")

if USE_CYTHON:
    ext_mods = cythonize(_ext_mods)
else:
    ext_mods = _ext_mods

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()
    
setup(
    name = "py-veo",
    version = "1.3.4",
    ext_modules = ext_mods,
    data_files = [("share/py-veo/examples", _example_files),
                  ("share/py-veo", ["README.md"])],
    author = "Erich Focht",
    author_email = "efocht@gmail.com",
    license = "GPLv2",
    requires = ["numpy"],
    description = "Python bindings for the VE Offloading API",
    url = "https://github.com/sx-aurora/py-veo",
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
      'Development Status :: 4 - Beta',
      'Environment :: Console',
      'Intended Audience :: Education',
      'Intended Audience :: Developers',
      'Intended Audience :: Science/Research',
      'License :: OSI Approved :: GNU General Public License (GPL)',
      'Operating System :: POSIX :: Linux',
      'Programming Language :: Python',
      'Topic :: Software Development :: Libraries :: Python Modules'
      ]
)



import veo
import os


print("\nVEO test:")
print("Pass a string on stack to VE function. Function mallocs a buffer")
print("and copies the string into the buffer. The VH program copies the")
print("buffer content back and prints it.")
print("\n")

p = veo.VeoProc(0)
lib = p.load_library(os.getcwd() + "/libvetest5.so")
c = p.open_context()

lib.malloc_buff.args_type("char *")
lib.malloc_buff.ret_type("void *")

req = lib.malloc_buff(c, veo.OnStack(b'Hello from VH!\0', 15))
print(req)
r = req.wait_result()
print("received address from VE: %r" % r)

buff = b'                                                       '
p.read_mem(buff, r, 15)

print(buff)

del p
print("finished")
